<?php
/**
 * @file
 * Include file for settings.php to block POST requests.
 */

define('POST_PROTECTOR_DEFAULT_KEY', 'post_protector');
define('POST_PROTECTOR_DEFAULT_VALUE', '4375892345');
define('POST_PROTECTOR_DEFAULT_HASH', '728347128394');

/**
 * Verifies that incoming POST requests are valid.
 */
function post_protector_check() {
  if ($_SERVER['REQUEST_METHOD'] == 'POST') {
    $settings = post_protector_get_config();
    $enabled = $settings['post_protector_enabled'];

    if ($enabled) {
      $post_protector_verification_key = $settings['post_protector_verification_key'];
      $post_protector_verification_value = post_protector_get_verification_value();

      $access_allowed = isset($_POST[$post_protector_verification_key])
        && ($_POST[$post_protector_verification_key] == $post_protector_verification_value);

      if (!$access_allowed) {
        post_protector_access_denied();
      }
    }
  }
}

/**
 * Retrieves the configuration for the post_protector module.
 *
 * @return array
 *   An associative array of the module configuration.
 */
function post_protector_get_config() {
  static $settings = array();

  if (empty($settings)) {
    global $conf;

    $defaults = array(
      'post_protector_enabled' => FALSE,
      'post_protector_enable_form_hashing' => FALSE,
      'post_protector_verification_key' => POST_PROTECTOR_DEFAULT_KEY,
      'post_protector_verification_value' => POST_PROTECTOR_DEFAULT_VALUE,
      'post_protector_hash_key' => POST_PROTECTOR_DEFAULT_HASH,
      'post_protector_error_code' => 403,
      'post_protector_header_response' => 'HTTP/1.0 403 Forbidden',
      'post_protector_error_body' => '<html><head><title>403 Forbidden</title></head><body><h1>POST Requests are forbidden.</h1></body></html>',
    );

    $overrides = array_intersect_key($conf, $defaults);
    $settings = array_merge($defaults, $overrides);
  }

  return $settings;
}

/**
 * Returns the configured access denied response.
 */
function post_protector_access_denied() {
  $settings = post_protector_get_config();

  $error_body = $settings['post_protector_error_body'];
  $error_code = $settings['post_protector_error_code'];
  $header_response = $settings['post_protector_header_response'];

  header($header_response, TRUE, $error_code);
  print($error_body);
  exit();

}

/**
 * Generates a value for POST verification.
 *
 * @param string $form_id
 *   The ID of the form.  Defaults to the POST form_id value if available.
 *
 * @return string
 *   The POST verification value.
 */
function post_protector_get_verification_value($form_id = '') {
  if (empty($form_id) && isset($_POST['form_id'])) {
    $form_id = $_POST['form_id'];
  }

  $settings = post_protector_get_config();
  $value = $settings['post_protector_verification_value'];
  $hash_key = $settings['post_protector_hash_key'];
  $form_hashing = $settings['post_protector_enable_form_hashing'];

  if ($form_hashing) {
    return drupal_hash_base64($form_id . ':' . $value . ':' . $hash_key);
  }
  else {
    return $value;
  }
}
